﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.ToolTip;

namespace AntiAircraftGun
{
    public partial class FormAntiAircraftGunConfig : Form
    {
        /// <summary>
        /// Переменная-выбранная машина
        /// </summary>
        DrawingAntiAircraftGun _antiAircraftGun = null;
        /// <summary>
        /// Событие
        /// </summary>
        private event Action<DrawingAntiAircraftGun> EventAddAntiAircraftGun;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormAntiAircraftGunConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
            buttonCancel.Click += (sender, e) => { Close(); };
        }
        /// <summary>
        /// Отрисовать орудие
        /// </summary>
        private void DrawAntiAircraftGun()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _antiAircraftGun?.SetPosition(5, 5, pictureBoxObject.Width, pictureBoxObject.Height);
            _antiAircraftGun?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev"></param>
        public void AddEvent(Action<DrawingAntiAircraftGun> ev)
        {
            if (EventAddAntiAircraftGun == null)
            {
                EventAddAntiAircraftGun = ev;
            }
            else
            {
                EventAddAntiAircraftGun += ev;
            }
        }
        ///
        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label).DoDragDrop((sender as Label).Name, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _antiAircraftGun = new DrawingAntiAircraftGun((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White);
                    break;
                case "labelModifiedObject":
                    _antiAircraftGun = new DrawingUpdateAntiAircraftGun((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, Color.Black,
                    checkBoxGun.Checked, checkBoxRadar.Checked);
                    break;
            }
            DrawAntiAircraftGun();
        }
        /// <summary>
        /// Отправляем цвет с панели
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Control).DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelBaseColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>labelModifiedObject
        /// Принимаем основной цвет
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LableBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_antiAircraftGun == null || !e.Data.GetDataPresent(typeof(Color)))
            {
                return;
            }
            _antiAircraftGun.AntiAircraftGun.BodyColor = (Color)e.Data.GetData(typeof(Color));
            DrawAntiAircraftGun();
        }
        /// <summary>
        /// Принимаем дополнительный цвет
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LableDopColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_antiAircraftGun == null || _antiAircraftGun.GetType() != typeof(DrawingUpdateAntiAircraftGun) || !e.Data.GetDataPresent(typeof(Color)))
            {
                return;
            }
            if (_antiAircraftGun.AntiAircraftGun is EntityUpdateAntiAircraftGun upGun)
            {
                upGun.DopColor = (Color)e.Data.GetData(typeof(Color));
            }
            DrawAntiAircraftGun();
        }
        /// <summary>
        /// Добавление орудия
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddAntiAircraftGun?.Invoke(_antiAircraftGun);
            Close();
        }
    }
}
